package calendarapp.gui;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.lang.annotation.Annotation;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JButton;

import calendarapp.ctrl.CalendarDayButtonAction;
import calendarapp.ctrl.ControllerFactory;
import calendarapp.plugins.CalendarPlugin;
import calendarapp.plugins.Coloured;
import calendarapp.plugins.Pluggable;


/**
 * This class describes a single clickable calendar button. Instances of this
 * class are added by CalendarMonth. This class loads plugins related to itself
 * upon initialization.
 * @author adamw
 *
 */
@SuppressWarnings("serial")
public class CalendarDayButton extends JButton implements Constrained {
	private GregorianCalendar date;
	List<String> annotationStrings = new LinkedList<String>();

	public CalendarDayButton(String label, GregorianCalendar date, List<Pluggable<?>> plugins) {
		super(label);

		this.date = date;
		setFont(new Font("SansSerif", Font.BOLD, 9));
		setMargin(new Insets(0,0,0,0));

		applyPlugins(plugins);

		// TODO H3: Replace the call below (ControllerFactory.getController(...)) with
		// Google Guice injection. NOTE: Make sure the returned object IS a singleton.
		addActionListener(ControllerFactory.getController(CalendarDayButtonAction.class));
	}

	private void colourButton(Pluggable<?> plugin) {
		if (Coloured.class.isAssignableFrom(plugin.getClass())) {
			Coloured coloured = (Coloured) plugin;
			this.setBackground(coloured.getColor());
		}
	}

	private void setAnnotationStrings(Pluggable<?> plugin) {
		Annotation[] annotations = plugin.getClass().getAnnotations();

		for (Annotation a : annotations) {
			if (CalendarPlugin.class.isAssignableFrom(a.getClass())) {
				CalendarPlugin calendarPlugin = (CalendarPlugin) a ;

				// TODO G1: Uncomment the code row below.
				//annotationStrings.add(calendarPlugin.value());
			}
		}
	}

	public List<String> getAnnotationStrings() {
		return annotationStrings;
	}

	private void applyPlugins(List<Pluggable<?>> plugins) {
		// TODO G2: Here, you apply the plugins to the button. What you need to do
		// is loop through the loaded plugins and do the following:
		// 1. Check if plugin.get() is of type GregorianCalendar, if so, do this:
		//    a. Check if date equals the date in the object returned by plugin.get()
		//       Be careful with GregorianCalendar.equals() though. It doesn't really
		//       work as you would expect (it checks to many fields).
		//    b. If the two dates match, call colourButton(plugin) and
		//       setAnnotationStrings(plugin).
		// 2. If not (1), then check if plugin.get() is of type Integer. The idea here
		//    is the same as in (1), except that you compare to a
		//    GregorianCalendar.DAY_OF_WEEK returned from the plugin. 
	}

	public static GridBagConstraints getGlobalConstraints() {
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.weighty = 1.0;
		constraints.weightx = 1.0;
		constraints.fill = GridBagConstraints.BOTH;
		constraints.insets = new Insets(0, 0, 0, 0);

		return constraints;
	}

	public GridBagConstraints getConstraints() {
		GridBagConstraints constraints = getGlobalConstraints();
		constraints.insets = new Insets(1, 1, 1, 1);

		return constraints;
	}
}
