package edu.chl.hajo.monopoly.core;

import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Space;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.core.space.Tax;

import java.util.Collection;

import java.util.List;
import java.util.Random;

/**
 * Class representing the overall game
 *
 * @author hajo
 *
 */
public class Monopoly implements IMonopoly {

    private final List<Player> players;
    private final DicePair dicePair;
    private final Board board;

    private Player actual;

    public Monopoly(List<Player> players, Board board, DicePair dicePair) {
        this.dicePair = dicePair;
        this.board = board;
        this.players = players;

        // Always create a new game when starting
        actual = players.get(new Random().nextInt(players.size()));
    }

    @Override
    public void next() {
        if (!actual.hasDebt()) {
            int i = players.indexOf(actual);
            actual = players.get((i + 1) % players.size());
            dicePair.setEnabled(true);
        }
    }

    @Override
    public void move() {
        Space oldPos = actual.getPosition();
        dicePair.roll();
        Space newPos = board.getSpace(oldPos, dicePair.getTotal());
        actual.setPosition(newPos);
        dicePair.setEnabled(dicePair.bothAre(6));

        if (board.hasPassedGo(oldPos, newPos)) {
            //System.out.println("passed go");
            actual.income(MonopolyOptions.getBonus());
        }

        checkForRent(newPos);
        checkForCard(newPos);
        checkForTax(newPos);

    }

    private void checkForTax(Space position) {
        if (position instanceof Tax) {   // Bad replace with visitor pattern TODO
            Tax t = (Tax) position;
            actual.addDebt(new Debt(null, t.getTax()));
        }
    }

    private void checkForRent(Space position) {
        if (position instanceof Street) {
            Street s = (Street) position;
            if (s.hasOwner() && s.getOwner() != actual) {
                actual.addDebt(new Debt(s.getOwner(), s.getRent()));
            }
        }
    }

    private void checkForCard(Space space) {
        // CardVisitor later

    }

    @Override
    public void sell(Space s) {
        if (s instanceof Property) {
            Property p = (Property) s;

            if (p.getOwner() == actual) {
                p.setOwner(Player.NONE);
                actual.income(p.getPrice());
            }
        }
    }

    @Override
    public void buy() {
        if (actual.getPosition() instanceof Property) {
            Property p = (Property) actual.getPosition();
            if (!p.hasOwner() && actual.getBalance() >= p.getPrice()) {
                p.setOwner(actual);
                actual.withDraw(p.getPrice());
            }
        }
    }

    @Override
    public void payDept() {
        actual.payDebt();
    }

    // ------- Accessors -------------
    @Override
    public Player getActual() {
        return actual;
    }

    @Override
    public Collection<Player> getPlayers() {
        return players;
    }

    @Override
    public DicePair getDicePair() {
        return dicePair;
    }

    @Override
    public Board getBoard() {
        return board;
    }

}
