package edu.chl.hajo.monopoly.core;

import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Space;
import java.util.ArrayList;
import java.util.List;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import org.junit.Before;
import org.junit.Test;

/**
 * 
 *
 * @author hajo
 */
public class TestBoard {

    Board b;    // Will be fresh before each test
    final int SIZE = 10;  // Test size

    // Create the fixture
    @Before  // Called before each test
    public void before() {
        List<Space> spaces = new ArrayList<>();
        for (int i = 0; i < SIZE; i++) {
            // Dummy names, index as name
            spaces.add(new Property(String.valueOf(i), i));  
        }
        b = new Board(spaces);  // Pass in dependencies

    }

    @Test
    public void testGetSpace() {
        // Check name for all
        for (int i = 0; i < b.size(); i++) {
            String name = String.valueOf(i);
            Space s = b.getSpace(name);
            assertTrue(s.getName().equals(name));
        }
    }

    @Test
    public void testGetSpaceSteps() {
        // No move
        Space s0 = b.getSpace(b.getStart(), 0);
        assertTrue(s0 == b.getStart());

        Space s1 = b.getSpace(b.getStart(), 1);
        Space s2 = b.getSpace(s1, 1);
        assertTrue(b.getSpace(b.getStart(), 2) == s2);

        // Completing a round, standing on last space
        Space s3 = b.getSpace(b.getStart(), SIZE - 1);
        Space s4 = b.getSpace(s3, 1);
        assertTrue(s4 == b.getStart());

    }

    @Test
    public void testHasPassedGo() {
        
        Space s1 = b.getSpace(b.getStart(), SIZE - 2);
        Space s2 = b.getSpace(s1, 1);
        assertFalse(b.hasPassedGo(s1, s2));

        Space s3 = b.getSpace(s1, 2);
        assertTrue( b.hasPassedGo(s1, s3));

    }

}
