
package edu.chl.hajo.monopoly.core;

import edu.chl.hajo.monopoly.core.space.Space;
import edu.chl.hajo.monopoly.event.Event;
import edu.chl.hajo.monopoly.event.EventBus;
import java.util.Collection;

/**
 * EventWrapper around Monopoly core
 * Applying Open closed principle
 *
 * @author hajo
 */
public class MonopolyEventWrapper implements IMonopoly {

    private final IMonopoly m;

    public MonopolyEventWrapper(IMonopoly m) {
        this.m = m;
    }

    @Override
    public void move() {
        EventBus.INSTANCE.publish(new Event(Event.Tag.REMOVE_PLAYER, m.getActual()));
        m.move();
        EventBus.INSTANCE.publish(new Event(Event.Tag.ADD_PLAYER, m.getActual()));
        EventBus.INSTANCE.publish(new Event(Event.Tag.DICE_RESULT, m.getDicePair()));
    }

    @Override
    public void next() {
        m.next();
        EventBus.INSTANCE.publish(new Event(Event.Tag.NEXT, m.getActual()));
    }

    @Override
    public void payDept() {
    }

    @Override
    public void sell(Space s) {
    }

    @Override
    public void buy() {
        m.buy();
        EventBus.INSTANCE.publish(new Event(Event.Tag.BUY, m.getActual()));
    }

    @Override
    public Player getActual() {
        return m.getActual();
    }

    @Override
    public Board getBoard() {
        return m.getBoard();
    }

    @Override
    public DicePair getDicePair() {
        return m.getDicePair();
    }

    @Override
    public Collection<Player> getPlayers() {
        return m.getPlayers();
    }
}
