package edu.chl.hajo.monopoly.gui;

import edu.chl.hajo.monopoly.core.IMonopoly;
import edu.chl.hajo.monopoly.ctrl.NewGameCtrl;
import edu.chl.hajo.monopoly.ctrl.SaveAsCtrl;
import edu.chl.hajo.monopoly.event.Event;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.event.IEventHandler;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.WindowConstants;

/**
 * Application main window
 *
 * @author hajo
 *
 */
@SuppressWarnings("serial")
public final class MainFrame extends JFrame implements IEventHandler {

    private static MainFrame instance;
    private boolean isRunning = false;  // Used to (dis)enable components 
    private IMonopoly mp;

    public static MainFrame getInstance() {
        if (instance == null) {
            instance = new MainFrame();
        }
        return instance;
    }

    private MainFrame() {
        EventBus.INSTANCE.register(this);
        this.setTitle("Monopoly");
        this.setLayout(new BorderLayout());
        this.addWindowListener(listener);
        this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        
        // Better use a GUI factory
        
        this.getContentPane().add(new PlayersPanel(), BorderLayout.SOUTH);
        this.getContentPane().add(new SpacesPanel(), BorderLayout.CENTER);
        this.getContentPane().add(new ButtonPanel(), BorderLayout.NORTH);
        initMenu();
        setSize(500, 300);
    }

    @Override
    public void onEvent(Event evt) {
        switch (evt.getTag()) {
            case NEW_GAME:
                this.mp = (IMonopoly) evt.getValue();
                break;
        }
    }

    // Handling shutdown -----------------
    private void exit() {
        System.exit(0);
    }

    private final WindowListener listener = new WindowAdapter() {
        @Override
        public void windowClosing(WindowEvent e) {
            exit();
        }
    };

    // ----- The menu ---------
    // Common listener for menu
    private final ActionListener menuListener = new ActionListener() {

        @Override
        public void actionPerformed(ActionEvent e) {
            Object s = e.getSource();
            if (s == fileExit) {
                MainFrame.this.exit();
            } else if (s == fileNew) {
                new NewGameCtrl().execute();
                setRunning(true);
                fileSaveAs.setEnabled(true); // Ugly just for now
                MainFrame.this.validate();
            } else if (s == aboutAbout) {
                JOptionPane.showMessageDialog(MainFrame.this, "Monopoly by Hajo");
            } else if (s == fileSave) {
                // TODO
            } else if (s == fileSaveAs) {
                new SaveAsCtrl(mp, MainFrame.this).execute();
            } else if (s == fileOpen) {
                // TODO
            }
        }
    };

    private final JMenuBar menuBar = new JMenuBar();
    private final JMenu file = new JMenu("File");
    private final JMenu about = new JMenu("About");
    private final JMenu options = new JMenu("Options");
    private final JMenuItem fileNew = new JMenuItem("New");
    private final JMenuItem fileSave = new JMenuItem("Save");
    private final JMenuItem fileSaveAs = new JMenuItem("Save as ...");
    private final JMenuItem fileOpen = new JMenuItem("Open");
    private final JMenuItem fileExit = new JMenuItem("Exit");
    private final JMenuItem aboutAbout = new JMenuItem("About");
    private final JMenuItem optionsPlayers = new JMenuItem("Players");

    private void initMenu() {
        this.setJMenuBar(menuBar);
        menuBar.add(file);
        menuBar.add(options);
        menuBar.add(about);
        file.add(fileNew);
        file.add(fileSave);
        file.add(fileSaveAs);
        file.add(fileOpen);
        about.add(aboutAbout);
        options.add(optionsPlayers);
        options.setEnabled(!isRunning());
        fileNew.addActionListener(menuListener);
        fileExit.addActionListener(menuListener);
        fileSave.addActionListener(menuListener);
        fileSaveAs.addActionListener(menuListener);
        fileOpen.addActionListener(menuListener);
        fileSave.setEnabled(isRunning());
        fileSaveAs.setEnabled(isRunning());
        aboutAbout.addActionListener(menuListener);
    }

    private void setRunning(boolean b) {
        isRunning = b;
    }

    private boolean isRunning() {
        return isRunning;
    }

}
