package edu.chl.hajo.monopoly;

import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.core.Game;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.view.MainFrame;

/**
 * ITERATION ONE 
 * 
 * GENERAL
 * We use general knowledge and a small subset of the model
 * to design and implement this iteration
 * 
 * - We use the "standard" (see NOTE) MVC design. Package view is V, package ctrl is C and package core is M.
 * 	    Observable: Player, Game
 * 	    Observer: MainFrame	
 * - View should never have any control code. The views have listeners. Listeners
 *   always runs a control (no other code except possible changes to GUI in listeners)
 * - The controls are implemented with the "Command pattern" i.e each command is a class
 *   with an execute method.
 * 
 * USE CASES
 * We pick two very central use cases
 * - Move actual player
 * - Switch actual player
 * 
 * DESIGN
 * This is a very minimal pre-design iteration, i.e. we just want something to run. 
 * After this iteration we'll do a lot of redesign.
 *
 * Some specific design choices for now (possible have to change)
 * 
 * - Central access point to the model (from controls/GUI) is the Game class (Singleton)
 * - To simplify handling of Spaces we would like to have them all in a single list.
 * - This will implicitly give the spaces an *ordering*!!! (Spaces don't have an index 
 *   attribute)
 * - GUI can create a view (list of SpacePanels) using the list. 
 * - Also reading data from a file into the list will be easy
 * 
 * UNIQUENESS
 * - Players will be uniquely identified by their Pieces
 * - Spaces have *no* unique id (implicit ordering, see above)
 * 
 * NOTE: Standard MVC in Java uses Java Beans style events (PropertyChangeListener etc.) to implement
 *   the "Observer pattern" (state changes in model are sent to GUI). State changes (names)are collected 
 *   in enum StateChanges. All observables implement the IObserver interface.
 * 
 * @author hajo
 *
 */
public class Main {

	public static void main(String[] args) {
		Game g = Game.getInstance();
		// Ordering matters! Used buy game
		g.addPlayer(new Player("Olle", Piece.DOG));
		g.addPlayer(new Player("Fia", Piece.TOP_HAT));
		g.addPlayer(new Player("Pelle", Piece.CAR));
		g.addPlayer(new Player("Sanna", Piece.IRON));
		new MainFrame().setVisible(true);
		g.startGame();
	}
}
