package edu.chl.hajo.monopoly.core;

import java.util.ArrayList;
import java.util.List;

import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.event.GameEvents;
import edu.chl.hajo.monopoly.event.StateChanges;
import edu.chl.hajo.monopoly.io.ChanceCommunityResources;
import edu.chl.hajo.monopoly.util.PlayerData;

public class Game2 implements IGame {

	private List<Player> players;
	private Player actualPlayer;
	private Dices dices;
	private Board2 board;
	private Rules rules;

	private EventBus eb = EventBus.getInstance();

	public Game2(Board2 board, Dices dices) {
		this.dices = dices;
		this.board = board;
	}
	
	// --------- IGame --------------
	
	@Override
	public void setPlayers(List<PlayerData> datum) {
		players = new ArrayList<Player>();
		for( PlayerData pd : datum){
			players.add(new Player(pd, rules, board, dices));
		}
		this.actualPlayer = players.get(0);
		for (Player p : players) {
			board.put(p);  // position == 0;
		}
		// To create the panels
		eb.broadcast(StateChanges.GAME_NEW_PLAYERS, null, players);
		// To mark the first player as the actual
		eb.send(actualPlayer, StateChanges.GAME_NEW_ACTUAL_PLAYER, null, true);
	}
	
	@Override
	public void endTurn() {
		int index = players.indexOf(actualPlayer);
		actualPlayer = players.get(++index % players.size());
		dices.reset();
		// Unset the active border for all
		eb.broadcast(StateChanges.GAME_NEW_ACTUAL_PLAYER, null, false);
		// Set active border for actual player
		eb.send(actualPlayer, StateChanges.GAME_NEW_ACTUAL_PLAYER, null, true);
	}

	@Override
	public void killPlayer() {
		Player toDie = actualPlayer;
		endTurn();
		board.remove(toDie);
		//toDie.kill();
		eb.broadcast(GameEvents.PLAYER_DIE, null, null);
	}

	@Override
	public void chanceCommunity() {
		String data = ChanceCommunityResources.getNextCardData();
		
		String action = data.split(":")[1];
		int arg = Integer.valueOf(data.split(":")[2]);

		// TODO move doesn't exist
		if (action.equals("move")) {
			actualPlayer.move(arg);
		} else if (action.equals("moveTo")) {
			actualPlayer.move(arg);
		} else if (action.equals("goJail")) {
			actualPlayer.goToJail();
		} else if (action.equals("withdraw")) {
			actualPlayer.withdraw(arg);
		} else if (action.equals("getFromAll")) {
			for (Player p : players) {
				p.withdraw(arg);
			}
			actualPlayer.addIncome(arg * players.size());
		} else {
			throw new IllegalArgumentException(action);
		}
	}

	@Override
	public IPlayer getPlayer() {
		return actualPlayer;
	}

	@Override
	public IRules getRules() {
		return rules;
	}
	
	@Override
	public void setRules(String ruleName) {
		// TODO look up the rules
		this.rules = new Rules();
	}
}
