package edu.chl.hajo.monopoly.core;

import java.util.List;

import edu.chl.hajo.monopoly.constants.GameEvents;
import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.event.EventBus;

public class Game2 implements IGame {
	
	private List<Player> players;
	private Player actualPlayer;
	private Dices dices;
	private Board2 board;
	
	private EventBus eb = EventBus.getInstance();
	
	public Game2(Board2 board, Dices dices) {
		this.dices = dices;
		this.board = board;
	}

	// ---------- IGame ---------------
	
	@Override
	public void setPlayers(List<Player> players) {
		this.players = players;
		this.actualPlayer = players.get(0); // TODO check param
		for (Player p : players) {
			board.getSpaces().get(0).addVisitingPlayer(p);
		}
		// To create the panels
		eb.broadcast(StateChanges.GAME_NEW_PLAYERS, null, players);	
		// To mark the first player as the actual
		eb.send(actualPlayer, StateChanges.GAME_NEW_ACTUAL_PLAYER, null, true);
		
	}
	
	@Override
	public List<AbstractSpace> getSpaces() {
		return board.getSpaces();
	}

	@Override
	public List<Player> getPlayers() {
		return players;
	}
	
	@Override
	public void setActualPlayer(Player actualPlayer) {
		this.actualPlayer = actualPlayer;
		// Unset the active border for all player panels
		eb.broadcast(StateChanges.GAME_NEW_ACTUAL_PLAYER, null, false);
		// Set active border for actual player panel
		eb.send(actualPlayer, StateChanges.GAME_NEW_ACTUAL_PLAYER, null, true);
	}

	@Override
	public Player getActualPlayer() {
		return actualPlayer;
	}

	@Override
	public AbstractSpace getActualSpace() {
		int pos = actualPlayer.getPosition();
		return board.getSpaces().get(pos);
	}

	@Override
	public Dices getDices() {
		return dices;
	}
	
	@Override
	public void message( String msg){
		eb.broadcast(GameEvents.MESSAGE, null, msg );
	}
}
