package edu.chl.hajo.monopoly.core.space;

import java.util.HashSet;
import java.util.Set;

import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.IDialogVisitor;
import edu.chl.hajo.monopoly.core.IPanelVisitor;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.event.EventBus;

/**
 * 
 * Base class for all spaces (streets, railroads, corners, jail, etc..)
 * Common code
 * 
 * @author hajo
 * 
 */
public abstract class AbstractSpace {

	protected EventBus eb = EventBus.getInstance();
	
	// All spaces must have a unique name
	protected String name;
	// Players on this space
	private Set<Player> visitingPlayers = new HashSet<Player>();
	
	// To be implemented by subclasses
	public abstract void visit( IPanelVisitor pf );
	public abstract void visit( IDialogVisitor df );
	// Spaces fires events when player arrives
	public abstract void checkEvent(Player player);
	
	public AbstractSpace(String name) {
		this.name = name;
	}
	
	public void addVisitingPlayer(Player p) {
		visitingPlayers.add(p);
		eb.send(this, StateChanges.SPACE_ADD_VISITING, null, p.getPiece());
		// Depending on sub class
		// Template pattern
		checkEvent(p);
	}

	public void removeVisitingPlayer(Player p) {
		visitingPlayers.remove(p);
		eb.send(this, StateChanges.SPACE_REMOVE_VISITING, null, p.getPiece());
	}

	public boolean isVisitingPlayer(Player p) {
		return visitingPlayers.contains(p);
	}

	public String getName() {
		return name;
	}

}
