package edu.chl.hajo.monopoly.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.ctrl.CF;
import edu.chl.hajo.monopoly.event.EventBus;

/**
 * Application main window
 * 
 * @author hajo
 */
@SuppressWarnings("serial")
public class MainFrame extends JFrame implements PropertyChangeListener {
	private JPanel jPnlWest;
	private JPanel jPnlNorth;
	private JPanel jPnlPlayers;
	private JPanel jPnlSouth;
	private JPanel jPnlCenter;
	private JPanel jPnlEast;
	private JMenuItem mnuINew;

	private final static float SCALE_FACTOR = 0.50f;

	public MainFrame(List<AbstractSpacePanel> spacePanels,
			DicesPanel dicesPanel, ControlPanel ctrPanel) {
		// Order matters
		initMenu();
		initGUI();
		initCtrl();
		initSpacePanels(spacePanels);
		initDicesPanel(dicesPanel);
		initControlPanel(ctrPanel);
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		String name = evt.getPropertyName();
		if (name.equals(StateChanges.GAME_NEW_PLAYERS.toString())) {
			/*
			 * This does some building of GUI. If a heavy
			 * transient GUI (much building) create helper class
			 */
			@SuppressWarnings("unchecked")
			List<Player> ps = (List<Player>) evt.getNewValue();
			List<PlayerPanel> panels = new ArrayList<PlayerPanel>();
			for (Player p : ps) {
				PlayerPanel pp = new PlayerPanel(p);
				panels.add(pp);
				EventBus.getInstance().connect(pp, p);
			}
			initPlayerPanels(panels);
		}
	}

	private void initCtrl() {
		mnuINew.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				CF.get(CF.Name.NEW_GAME).execute();
			}
		});
	}

	private void initMenu() {
		JMenuBar mb = new JMenuBar();
		JMenu file = new JMenu("file");
		mnuINew = new JMenuItem("new");
		file.add(mnuINew);
		file.add(new JMenuItem("exit"));
		JMenu options = new JMenu("options");
		options.add(new JMenuItem("players"));
		options.add(new JMenuItem("colors"));
		options.add(new JMenuItem("language"));
		JMenu about = new JMenu("about");
		about.add(new JMenuItem("about"));
		mb.add(file);
		mb.add(options);
		mb.add(about);
		this.setJMenuBar(mb);
	}

	private void initSpacePanels(List<AbstractSpacePanel> spacePanels) {
		// TODO Hard coded
		for (int i = 0; i < 11; i++) {
			jPnlNorth.add(spacePanels.get(i));
		}
		for (int i = 11; i < 20; i++) {
			jPnlEast.add(spacePanels.get(i));
		}
		for (int i = 30; i > 19; i--) {
			jPnlSouth.add(spacePanels.get(i));
		}
		for (int i = 39; i > 30; i--) {
			jPnlWest.add(spacePanels.get(i));
		}
	}

	private void initPlayerPanels(List<PlayerPanel> playerPanels) {
		jPnlPlayers = new JPanel();
		jPnlPlayers.setBackground(Color.LIGHT_GRAY);
		jPnlCenter.add(jPnlPlayers, BorderLayout.SOUTH);
		for (PlayerPanel p : playerPanels) {
			jPnlPlayers.add(p);
		}
		repaint();
	}

	private void initControlPanel(ControlPanel ctrlPanel) {
		jPnlCenter.add(ctrlPanel, BorderLayout.NORTH);
	}

	private void initDicesPanel(DicesPanel dicesPanel) {
		jPnlCenter.add(dicesPanel, BorderLayout.CENTER);
	}

	// --------- GUI ---------------

	private void initGUI() {
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		int size = Math.round(SCALE_FACTOR * Math.max(dim.width, dim.height));
		this.setSize(size, size);
		this.setLocationRelativeTo(null);
		this.setTitle("Monopoly");
		getContentPane().setLayout(new BorderLayout());
		this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		{
			jPnlCenter = new JPanel();
			BorderLayout jPnlCenterLayout = new BorderLayout();
			jPnlCenter.setLayout(jPnlCenterLayout);
			getContentPane().add(jPnlCenter, BorderLayout.CENTER);
		}

		{
			jPnlWest = new JPanel();
			BoxLayout jPnlWestLayout = new BoxLayout(jPnlWest,
					javax.swing.BoxLayout.Y_AXIS);
			jPnlWest.setLayout(jPnlWestLayout);
			getContentPane().add(jPnlWest, BorderLayout.WEST);
		}
		{
			jPnlNorth = new JPanel();
			BoxLayout jPnlNorthLayout = new BoxLayout(jPnlNorth,
					javax.swing.BoxLayout.X_AXIS);
			jPnlNorth.setLayout(jPnlNorthLayout);
			getContentPane().add(jPnlNorth, BorderLayout.NORTH);
		}
		{
			jPnlEast = new JPanel();
			BoxLayout jPnlEastLayout = new BoxLayout(jPnlEast,
					javax.swing.BoxLayout.Y_AXIS);
			jPnlEast.setLayout(jPnlEastLayout);
			getContentPane().add(jPnlEast, BorderLayout.EAST);
		}
		{
			jPnlSouth = new JPanel();
			BoxLayout jPnlSouthLayout = new BoxLayout(jPnlSouth,
					javax.swing.BoxLayout.X_AXIS);
			jPnlSouth.setLayout(jPnlSouthLayout);
			getContentPane().add(jPnlSouth, BorderLayout.SOUTH);
		}
	}

}
