package edu.chl.hajo.monopoly.view.dialog;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import edu.chl.hajo.monopoly.constants.GameEvents;
import edu.chl.hajo.monopoly.core.IDialogVisitor;
import edu.chl.hajo.monopoly.core.Locator;
import edu.chl.hajo.monopoly.core.space.NonProperty;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.core.space.Tax;
import edu.chl.hajo.monopoly.event.EventBus;


/**
 * Handle and display GUI dialogs  
 * 
 * @author hajo
 * 
 */

public class DialogHandler implements PropertyChangeListener {

	private DialogVisitor df;
	
	public DialogHandler(JFrame mainFrame) {
		df = new DialogVisitor(mainFrame);
		EventBus.getInstance().connect(this);
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		String p = evt.getPropertyName();
		
		boolean b = p.equals(GameEvents.MUST_PAY_RENT.toString())
				|| p.equals(GameEvents.MUST_PAY_TAX.toString())
				|| p.equals(GameEvents.GO_JAIL.toString())
				|| p.equals(GameEvents.CHANCE.toString())
				|| p.equals(GameEvents.COMMUNITY.toString());

		if (b) {
			Locator.getGame().getActualSpace().visit(df);
			df.getDialog().setVisible(true);
		} else if (p.equals(GameEvents.PLAYER_MUST_SELL.toString())) {
			Integer amount = (Integer) evt.getNewValue();
			new MustSellDialog(null, amount).setVisible(true);
		} else if (p.equals(GameEvents.PLAYER_DIE.toString())) {
			// TODO some message to the player
		} else if (p.equals(GameEvents.GAME_ENDED.toString())) {
			// TODO some message who win
		} else if (p.equals(GameEvents.MESSAGE.toString())){
			String msg = (String) evt.getNewValue();
			JOptionPane.showMessageDialog(null, msg, "Warning", JOptionPane.WARNING_MESSAGE);
		}
	}
	
	// Part of Visitor 
	private static class DialogVisitor implements IDialogVisitor {

		private AbstractEventDialog ed;
		private JFrame parent;	
		
		public DialogVisitor(JFrame parent) {
			this.parent = parent;
		}

		public AbstractEventDialog getDialog(){
			return ed;
		}
		
		@Override
		public void setDialog(Property p) {
			ed = new PropertyPayRentDialog(parent, p);
		}
		
		@Override
		public void setDialog(NonProperty p) {
			ed = new NonPropertyEventDialog(parent, p);
		}

		@Override
		public void setDialog(Street s) {
			ed = new StreetPayRentDialog(parent,s);
		}

		@Override
		public void setDialog(Tax t) {
			ed = new PayTaxDialog(parent,t);
		}
	}
}
