package edu.chl.hajo.monopoly;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import edu.chl.hajo.monopoly.build.GameBuilder;
import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.core.IGame;
import edu.chl.hajo.monopoly.core.Locator;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.view.dialog.MustSellDialog;

/**
 * This is semi-automated testing. Have to click in dialogs.
 * @author hajo
 *
 */
public class TestDialogs {
	
	static IGame g;
	
	@BeforeClass
	public static void setUpClass() throws Exception {
		// Must be first
		EventBus.getInstance().on(false);
		g = new GameBuilder().build();
		Locator.set(g);
	}
	
	@Before
	public void setUp(){
		List<Player> ps = new ArrayList<Player>();
		ps.add(new Player("Olle", Piece.DOG));
		ps.add(new Player("Fia", Piece.TOP_HAT));
		ps.add(new Player("Pelle", Piece.CAR));
		ps.add(new Player("Sanna", Piece.IRON));
		g.setPlayers(ps);
	}

	@Test
	public void testMustSellDialogSell() {
		List<AbstractSpace> sp = Locator.getGame().getSpaces();
		// Get some streets
		Street s1 = (Street) sp.get(1);
		Street s2 = (Street) sp.get(3);
		Street s3 = (Street) sp.get(6);
		
		s1.setPrice(100);
		s2.setPrice(100);
		s3.setRent(150); 
		
		List<Player> ps = Locator.getGame().getPlayers();
		
		Player olle = ps.get(0);
		olle.setBalance(0);
		olle.addProperty(s1);
		olle.addProperty(s2);
		s1.setOwner(olle);
		s2.setOwner(olle);
		
		Player fia = ps.get(1);
		fia.setBalance(0);
		s3.setOwner(fia);
		fia.addProperty(s3);
			
		new MustSellDialog(null, s3.getRent()).setVisible(true);
		
		Assert.assertTrue( !s1.hasOwner());
		Assert.assertTrue( !s2.hasOwner());
		Assert.assertTrue(olle.getBalance() == 50);
		// Note: Can't check Fia here
		
	}
	
	@Test
	public void testMustSellDialogBankrupcy() {
		List<AbstractSpace> sp = Locator.getGame().getSpaces();
		Street s1 = (Street) sp.get(1);
		Street s2 = (Street) sp.get(3);
		Street s3 = (Street) sp.get(6);
		
		s1.setPrice(100);
		s2.setPrice(100);
		s3.setRent(500);  // Must die 
		
		List<Player> ps = Locator.getGame().getPlayers();
		
		Player olle = ps.get(0);
		olle.setBalance(0);
		olle.addProperty(s1);
		olle.addProperty(s2);
		s1.setOwner(olle);
		s2.setOwner(olle);
		
		Player fia = ps.get(1);
		fia.setBalance(0);
		s3.setOwner(fia);
		fia.addProperty(s3);
			
		new MustSellDialog(null, s3.getRent()).setVisible(true);
		
		Assert.assertTrue(!s1.hasOwner());
		Assert.assertTrue(!s2.hasOwner());
		Assert.assertTrue(! ps.contains( olle) );
	}
}
